@@ -1,3 +1,6 @@
+#include <cstdint>
+#include <cmath>
+
 #include "BinFinder.h"
 
 #include "GenomeTrackFixedBin.h"
@@ -11,12 +14,6 @@
 #include "TrackExpressionFixedBinIterator.h"
 #include "TrackExpressionScanner.h"
 
-#ifdef __GXX_EXPERIMENTAL_CXX0X__
-#ifndef isnan
-#define isnan ::isnan
-#endif
-#endif
-
 using namespace std;
 using namespace rdb;
 
@@ -28,7 +25,7 @@ static double get_bin_tranformed_value(unsigned numexpr, bool force_binning, SEX
 	for (unsigned i = 0; i < numexpr; ++i) {
 		double val = scanner.last_real(i);
 
-		if (isnan(val)) {
+		if (std::isnan(val)) {
 			nan = true;
 			break;
 		} else {
@@ -51,7 +48,7 @@ static double get_bin_tranformed_value(unsigned numexpr, bool force_binning, SEX
 	if (nan)
 		value = numeric_limits<double>::quiet_NaN();
 	else {
-		if (index < 0 || (int)index >= length(lookup_table))
+		if ((int)index >= length(lookup_table))
 			verror("Internal error: index %d is out of range", index);
 
 		value = isReal(lookup_table) ? REAL(lookup_table)[index] : INTEGER(lookup_table)[index];
@@ -77,17 +74,19 @@ static SEXP build_rintervals_bintransform(GIntervalsFetcher1D *out_intervals1d,
 	}
 
 	SEXP rvals;
-	rprotect(rvals = allocVector(REALSXP, values.size()));
-	SET_VECTOR_ELT(answer, num_interv_cols + VALUE, rvals);
-	for (unsigned i = 0; i < values.size(); ++i)
-		REAL(rvals)[i] = values[i];
+	rprotect(rvals = RSaneAllocVector(REALSXP, values.size()));
+
+    for (unsigned i = 0; i < values.size(); ++i)
+        REAL(rvals)[i] = values[i];
+
+    SET_VECTOR_ELT(answer, num_interv_cols + VALUE, rvals);
 
 	SEXP col_names = getAttrib(answer, R_NamesSymbol);
 	SET_STRING_ELT(col_names, num_interv_cols + VALUE, mkChar("value"));
 
 	if (interv_ids) {
 		SEXP ids;
-		rprotect(ids = allocVector(INTSXP, interv_ids->size()));
+		rprotect(ids = RSaneAllocVector(INTSXP, interv_ids->size()));
 		for (vector<unsigned>::const_iterator iid = interv_ids->begin(); iid != interv_ids->end(); ++iid)
 			INTEGER(ids)[iid - interv_ids->begin()] = *iid;
 		SET_VECTOR_ELT(answer, num_interv_cols + ID, ids);
@@ -159,8 +158,8 @@ SEXP gbintransform(SEXP _intervals, SEXP _track_exprs, SEXP _breaks, SEXP _inclu
 		GIntervalsFetcher1D *intervals1d = NULL;
 		GIntervalsFetcher2D *intervals2d = NULL;
 		iu.convert_rintervs(_intervals, &intervals1d, &intervals2d);
-		auto_ptr<GIntervalsFetcher1D> intervals1d_guard(intervals1d);
-		auto_ptr<GIntervalsFetcher2D> intervals2d_guard(intervals2d);
+		unique_ptr<GIntervalsFetcher1D> intervals1d_guard(intervals1d);
+		unique_ptr<GIntervalsFetcher2D> intervals2d_guard(intervals2d);
 		intervals1d->sort();
 		intervals2d->sort();
 		intervals2d->verify_no_overlaps(iu.get_chromkey());
@@ -173,7 +172,7 @@ SEXP gbintransform(SEXP _intervals, SEXP _track_exprs, SEXP _breaks, SEXP _inclu
 		vector<GIntervalsBigSet2D::ChromStat> chromstats2d;
 		GInterval last_scope_interval1d;
 		GInterval2D last_scope_interval2d;
-		size_t size;
+		uint64_t size;
 		char error_prefix[1000];
 
 		TrackExprScanner scanner(iu);
@@ -304,7 +303,7 @@ SEXP gbintransform(SEXP _intervals, SEXP _track_exprs, SEXP _breaks, SEXP _inclu
 					iu.verify_max_data_size(out_values.size(), "Result");
 				}
 
-				size_t num_intervals = scanner.get_iterator()->is_1d() ? out_intervals1d.size() : out_intervals2d.size();
+				uint64_t num_intervals = scanner.get_iterator()->is_1d() ? out_intervals1d.size() : out_intervals2d.size();
 
 				void *result = allocate_res(num_intervals);
 
@@ -325,7 +324,7 @@ SEXP gbintransform(SEXP _intervals, SEXP _track_exprs, SEXP _breaks, SEXP _inclu
 			// collect results from kids
 			for (int i = 0; i < get_num_kids(); ++i) {
 				void *ptr = get_kid_res(i);
-				size_t num_intervals = get_kid_res_size(i);
+				uint64_t num_intervals = get_kid_res_size(i);
 
 				if (!num_intervals)
 					continue;
@@ -430,7 +429,9 @@ SEXP gbintransform(SEXP _intervals, SEXP _track_exprs, SEXP _breaks, SEXP _inclu
 		return answer;
 	} catch (TGLException &e) {
 		rerror("%s", e.msg());
-	}
+    } catch (const bad_alloc &e) {
+        rerror("Out of memory");
+    }
 	return R_NilValue;
 }
 
@@ -511,7 +512,7 @@ SEXP gtrack_bintransform(SEXP _track, SEXP _track_exprs, SEXP _breaks, SEXP _inc
 					if (cur_chromid != scanner.last_interval1d().chromid) {
 						cur_chromid = scanner.last_interval1d().chromid;
 						created_chromids.insert(cur_chromid);
-						sprintf(filename, "%s/%s", dirname.c_str(), iu.id2chrom(cur_chromid).c_str());
+						snprintf(filename, sizeof(filename), "%s/%s", dirname.c_str(), iu.id2chrom(cur_chromid).c_str());
 
 						if (itr_type == TrackExpressionIteratorBase::FIXED_BIN)
 							fixed_bin_track.init_write(filename, ((TrackExpressionFixedBinIterator *)scanner.get_iterator())->get_bin_size(), cur_chromid);
@@ -531,7 +532,7 @@ SEXP gtrack_bintransform(SEXP _track, SEXP _track_exprs, SEXP _breaks, SEXP _inc
 					// some of the chromosome could be previously skipped; we still must create them even if they are empty
 					for (GIntervals::const_iterator iinterv = all_genome_intervs1d.begin(); iinterv != all_genome_intervs1d.end(); ++iinterv) {
 						if (created_chromids.find(iinterv->chromid) == created_chromids.end()) {
-							sprintf(filename, "%s/%s", dirname.c_str(), iu.id2chrom(iinterv->chromid).c_str());
+							snprintf(filename, sizeof(filename), "%s/%s", dirname.c_str(), iu.id2chrom(iinterv->chromid).c_str());
 							sparse_track.init_write(filename, iinterv->chromid);
 						}
 					}
@@ -551,7 +552,7 @@ SEXP gtrack_bintransform(SEXP _track, SEXP _track_exprs, SEXP _breaks, SEXP _inc
 
 						cur_chromid1 = interv.chromid1();
 						cur_chromid2 = interv.chromid2();
-						sprintf(filename, "%s/%s", dirname.c_str(), GenomeTrack::get_2d_filename(iu.get_chromkey(), cur_chromid1, cur_chromid2).c_str());
+						snprintf(filename, sizeof(filename), "%s/%s", dirname.c_str(), GenomeTrack::get_2d_filename(iu.get_chromkey(), cur_chromid1, cur_chromid2).c_str());
 
 						qtree.reset(0, 0, iu.get_chromkey().get_chrom_size(cur_chromid1), iu.get_chromkey().get_chrom_size(cur_chromid2));
 						gtrack.init_write(filename, cur_chromid1, cur_chromid2);
@@ -563,14 +564,23 @@ SEXP gtrack_bintransform(SEXP _track, SEXP _track_exprs, SEXP _breaks, SEXP _inc
 
 				if (gtrack.opened())
 					gtrack.write(qtree);
-			} else
-				verror("Iterator type %s is not supported by the function", TrackExpressionIteratorBase::TYPE_NAMES[itr_type]);
+			} else {
+				size_t numTypeNames = sizeof(TrackExpressionIteratorBase::TYPE_NAMES) / sizeof(TrackExpressionIteratorBase::TYPE_NAMES[0]);
+				if (itr_type >= 0 && itr_type < numTypeNames) {
+					verror("Iterator type %s is not supported by the function", TrackExpressionIteratorBase::TYPE_NAMES[itr_type]);
+				} else {
+					verror("Invalid iterator type encountered");
+				}
+			}
+				
 		} catch (TGLException &e) {
 			verror("Error writing %s: %s", filename, e.msg());
 		}
 	} catch (TGLException &e) {
 		rerror("%s", e.msg());
-	}
+    } catch (const bad_alloc &e) {
+        rerror("Out of memory");
+    }
 	return R_NilValue;
 }
 
