@@ -5,6 +5,7 @@
  *      Author: hoichman
  */
 
+#include <cstdint>
 #include <fstream>
 #include <iomanip>
 #include <iostream>
@@ -37,10 +38,10 @@ static SEXP build_rintervals_extract(GIntervalsFetcher1D *out_intervals1d, GInte
 
 	for (unsigned iexpr = 0; iexpr < num_exprs; ++iexpr) {
 		SEXP expr_vals;
-		rprotect(expr_vals = allocVector(REALSXP, values[iexpr].size()));
-		SET_VECTOR_ELT(answer, num_interv_cols + iexpr, expr_vals);
+		rprotect(expr_vals = RSaneAllocVector(REALSXP, values[iexpr].size()));
 		for (unsigned i = 0; i < values[iexpr].size(); ++i)
 			REAL(expr_vals)[i] = values[iexpr][i];
+        SET_VECTOR_ELT(answer, num_interv_cols + iexpr, expr_vals);
 	}
 
 	SEXP col_names = getAttrib(answer, R_NamesSymbol);
@@ -53,7 +54,7 @@ static SEXP build_rintervals_extract(GIntervalsFetcher1D *out_intervals1d, GInte
 
 	if (interv_ids) {
 		SEXP ids;
-		rprotect(ids = allocVector(INTSXP, interv_ids->size()));
+		rprotect(ids = RSaneAllocVector(INTSXP, interv_ids->size()));
 		for (vector<unsigned>::const_iterator iid = interv_ids->begin(); iid != interv_ids->end(); ++iid)
 			INTEGER(ids)[iid - interv_ids->begin()] = *iid;
 		SET_VECTOR_ELT(answer, num_interv_cols + num_exprs, ids);
@@ -67,7 +68,7 @@ static SEXP build_rintervals_extract(GIntervalsFetcher1D *out_intervals1d, GInte
 
 extern "C" {
 
-SEXP gextract(SEXP _intervals, SEXP _exprs, SEXP _colnames, SEXP _iterator_policy, SEXP _band, SEXP _file, SEXP _intervals_set_out, SEXP _envir)
+SEXP C_gextract(SEXP _intervals, SEXP _exprs, SEXP _colnames, SEXP _iterator_policy, SEXP _band, SEXP _file, SEXP _intervals_set_out, SEXP _envir)
 {
 	try {
 		RdbInitializer rdb_init;
@@ -99,8 +100,8 @@ SEXP gextract(SEXP _intervals, SEXP _exprs, SEXP _colnames, SEXP _iterator_polic
 		GIntervalsFetcher1D *intervals1d = NULL;
 		GIntervalsFetcher2D *intervals2d = NULL;
 		iu.convert_rintervs(_intervals, &intervals1d, &intervals2d);
-		auto_ptr<GIntervalsFetcher1D> intervals1d_guard(intervals1d);
-		auto_ptr<GIntervalsFetcher2D> intervals2d_guard(intervals2d);
+		unique_ptr<GIntervalsFetcher1D> intervals1d_guard(intervals1d);
+		unique_ptr<GIntervalsFetcher2D> intervals2d_guard(intervals2d);
 		intervals1d->sort();
 		intervals2d->sort();
 		intervals2d->verify_no_overlaps(iu.get_chromkey());
@@ -167,7 +168,7 @@ SEXP gextract(SEXP _intervals, SEXP _exprs, SEXP _colnames, SEXP _iterator_polic
 			vector<GIntervalsBigSet2D::ChromStat> chromstats2d;
 			GInterval last_scope_interval1d;
 			GInterval2D last_scope_interval2d;
-			size_t size;
+			uint64_t size;
 			char error_prefix[1000];
 
 			if (is_1d_iterator)
@@ -266,7 +267,9 @@ SEXP gextract(SEXP _intervals, SEXP _exprs, SEXP _colnames, SEXP _iterator_polic
 		return answer;
 	} catch (TGLException &e) {
 		rerror("%s", e.msg());
-	}
+    } catch (const bad_alloc &e) {
+        rerror("Out of memory");
+    }
 	return R_NilValue;
 }
 
@@ -296,7 +299,7 @@ SEXP gextract_multitask(SEXP _intervals, SEXP _exprs, SEXP _colnames, SEXP _iter
 
 		const char *filename = isNull(_file) ? NULL : CHAR(STRING_ELT(_file, 0));
 		unsigned num_exprs = (unsigned)length(_exprs);
-		size_t num_intervals;
+		uint64_t num_intervals;
 		GIntervals out_intervals1d;
 		GIntervals2D out_intervals2d;
 		vector<unsigned> interv_ids;
@@ -306,8 +309,8 @@ SEXP gextract_multitask(SEXP _intervals, SEXP _exprs, SEXP _colnames, SEXP _iter
 		GIntervalsFetcher1D *intervals1d = NULL;
 		GIntervalsFetcher2D *intervals2d = NULL;
 		iu.convert_rintervs(_intervals, &intervals1d, &intervals2d);
-		auto_ptr<GIntervalsFetcher1D> intervals1d_guard(intervals1d);
-		auto_ptr<GIntervalsFetcher2D> intervals2d_guard(intervals2d);
+		unique_ptr<GIntervalsFetcher1D> intervals1d_guard(intervals1d);
+		unique_ptr<GIntervalsFetcher2D> intervals2d_guard(intervals2d);
 		intervals1d->sort();
 		intervals2d->sort();
 		intervals2d->verify_no_overlaps(iu.get_chromkey());
@@ -389,7 +392,7 @@ SEXP gextract_multitask(SEXP _intervals, SEXP _exprs, SEXP _colnames, SEXP _iter
 				TrackExprScanner scanner(iu);
 				GInterval last_scope_interval1d;
 				GInterval2D last_scope_interval2d;
-				size_t size;
+				uint64_t size;
 				char error_prefix[1000];
 
 				scanner.begin(_exprs, kid_intervals1d, kid_intervals2d, _iterator_policy, _band);
@@ -565,14 +568,14 @@ SEXP gextract_multitask(SEXP _intervals, SEXP _exprs, SEXP _colnames, SEXP _iter
 
 			for (unsigned iexpr = 0; iexpr < num_exprs; ++iexpr) {
 				SEXP expr_vals;
-				rprotect(expr_vals = allocVector(REALSXP, values[iexpr].size()));
-				SET_VECTOR_ELT(answer, num_interv_cols + iexpr, expr_vals);
+				rprotect(expr_vals = RSaneAllocVector(REALSXP, values[iexpr].size()));
 				for (unsigned i = 0; i < values[iexpr].size(); ++i)
 					REAL(expr_vals)[i] = values[iexpr][i];
+                SET_VECTOR_ELT(answer, num_interv_cols + iexpr, expr_vals);
 			}
 
 			SEXP ids;
-			rprotect(ids = allocVector(INTSXP, interv_ids.size()));
+			rprotect(ids = RSaneAllocVector(INTSXP, interv_ids.size()));
 			for (vector<unsigned>::const_iterator iid = interv_ids.begin(); iid != interv_ids.end(); ++iid)
 				INTEGER(ids)[iid - interv_ids.begin()] = *iid;
 			SET_VECTOR_ELT(answer, num_interv_cols + num_exprs, ids);
@@ -590,7 +593,9 @@ SEXP gextract_multitask(SEXP _intervals, SEXP _exprs, SEXP _colnames, SEXP _iter
 		}
 	} catch (TGLException &e) {
 		rerror("%s", e.msg());
-	}
+    } catch (const bad_alloc &e) {
+        rerror("Out of memory");
+    }
 	rreturn(R_NilValue);
 }
 
