@@ -5,7 +5,8 @@
  *      Author: hoichman
  */
 
-#include <math.h>
+#include <cstdint>
+#include <cmath>
 
 #include "rdbinterval.h"
 #include "rdbutils.h"
@@ -15,12 +16,6 @@
 #include "GIntervalsBigSet2D.h"
 #include "TrackExpressionScanner.h"
 
-#ifdef __GXX_EXPERIMENTAL_CXX0X__
-#ifndef isnan
-#define isnan ::isnan
-#endif
-#endif
-
 using namespace std;
 using namespace rdb;
 
@@ -43,7 +38,7 @@ struct IntervalSummary {
 
 	void update(double v) {
 		++num_bins;
-		if (!isnan(v)) {
+		if (!std::isnan(v)) {
 			num_non_nan_bins++;
 			total += v;
 			minval = min(minval, v);
@@ -79,7 +74,7 @@ static SEXP build_rintervals_summary(GIntervalsFetcher1D *intervals1d, GInterval
 {
 	SEXP answer, rsummary[NUM_COLS], colnames;
 	unsigned num_interv_cols;
-	size_t num_intervs;
+	uint64_t num_intervs;
 
 	if (intervals1d) {
 		num_interv_cols = GInterval::NUM_COLS;
@@ -93,11 +88,8 @@ static SEXP build_rintervals_summary(GIntervalsFetcher1D *intervals1d, GInterval
 
 	colnames = getAttrib(answer, R_NamesSymbol);
 
-	for (unsigned icol = 0; icol < NUM_COLS; ++icol) {
-		rprotect(rsummary[icol] = allocVector(REALSXP, num_intervs));
-		SET_VECTOR_ELT(answer, num_interv_cols + icol, rsummary[icol]);
-		SET_STRING_ELT(colnames, num_interv_cols + icol, mkChar(IntervalSummaryColNames[icol]));
-	}
+	for (unsigned icol = 0; icol < NUM_COLS; ++icol)
+		rprotect(rsummary[icol] = RSaneAllocVector(REALSXP, num_intervs));
 
 	for (unsigned i = 0; i < num_intervs; i++) {
 		REAL(rsummary[TOTAL_BINS])[i] = summaries[i].num_bins;
@@ -109,6 +101,11 @@ static SEXP build_rintervals_summary(GIntervalsFetcher1D *intervals1d, GInterval
 		REAL(rsummary[STDEV])[i] = summaries[i].num_non_nan_bins > 1 ? summaries[i].get_stdev() : numeric_limits<double>::quiet_NaN();
 	}
 
+    for (unsigned icol = 0; icol < NUM_COLS; ++icol) {
+        SET_VECTOR_ELT(answer, num_interv_cols + icol, rsummary[icol]);
+        SET_STRING_ELT(colnames, num_interv_cols + icol, mkChar(IntervalSummaryColNames[icol]));
+    }
+
 	return answer;
 }
 
@@ -127,8 +124,8 @@ SEXP gtracksummary(SEXP _expr, SEXP _intervals, SEXP _iterator_policy, SEXP _ban
 		GIntervalsFetcher1D *intervals1d = NULL;
 		GIntervalsFetcher2D *intervals2d = NULL;
 		iu.convert_rintervs(_intervals, &intervals1d, &intervals2d);
-		auto_ptr<GIntervalsFetcher1D> intervals1d_guard(intervals1d);
-		auto_ptr<GIntervalsFetcher2D> intervals2d_guard(intervals2d);
+		unique_ptr<GIntervalsFetcher1D> intervals1d_guard(intervals1d);
+		unique_ptr<GIntervalsFetcher2D> intervals2d_guard(intervals2d);
 		intervals1d->sort();
 		intervals1d->unify_overlaps();
 		intervals2d->sort();
@@ -143,8 +140,8 @@ SEXP gtracksummary(SEXP _expr, SEXP _intervals, SEXP _iterator_policy, SEXP _ban
 		SEXP answer;
 		SEXP colnames;
 
-		rprotect(answer = allocVector(REALSXP, NUM_COLS));
-		rprotect(colnames = allocVector(STRSXP, NUM_COLS));
+		rprotect(answer = RSaneAllocVector(REALSXP, NUM_COLS));
+		rprotect(colnames = RSaneAllocVector(STRSXP, NUM_COLS));
 
 		REAL(answer)[TOTAL_BINS] = summary.num_bins;
 		REAL(answer)[TOTAL_NAN_BINS] = summary.num_bins - summary.num_non_nan_bins;
@@ -162,7 +159,9 @@ SEXP gtracksummary(SEXP _expr, SEXP _intervals, SEXP _iterator_policy, SEXP _ban
 		return answer;
 	} catch (TGLException &e) {
 		rerror("%s", e.msg());
-	}
+    } catch (const bad_alloc &e) {
+        rerror("Out of memory");
+    }
 	return R_NilValue;
 }
 
@@ -179,8 +178,8 @@ SEXP gtracksummary_multitask(SEXP _expr, SEXP _intervals, SEXP _iterator_policy,
 		GIntervalsFetcher1D *intervals1d = NULL;
 		GIntervalsFetcher2D *intervals2d = NULL;
 		iu.convert_rintervs(_intervals, &intervals1d, &intervals2d);
-		auto_ptr<GIntervalsFetcher1D> intervals1d_guard(intervals1d);
-		auto_ptr<GIntervalsFetcher2D> intervals2d_guard(intervals2d);
+		unique_ptr<GIntervalsFetcher1D> intervals1d_guard(intervals1d);
+		unique_ptr<GIntervalsFetcher2D> intervals2d_guard(intervals2d);
 		intervals1d->sort();
 		intervals1d->unify_overlaps();
 		intervals2d->sort();
@@ -212,8 +211,8 @@ SEXP gtracksummary_multitask(SEXP _expr, SEXP _intervals, SEXP _iterator_policy,
 			SEXP answer;
 			SEXP colnames;
 
-			rprotect(answer = allocVector(REALSXP, NUM_COLS));
-			rprotect(colnames = allocVector(STRSXP, NUM_COLS));
+			rprotect(answer = RSaneAllocVector(REALSXP, NUM_COLS));
+			rprotect(colnames = RSaneAllocVector(STRSXP, NUM_COLS));
 
 			REAL(answer)[TOTAL_BINS] = summary.num_bins;
 			REAL(answer)[TOTAL_NAN_BINS] = summary.num_bins - summary.num_non_nan_bins;
@@ -232,7 +231,9 @@ SEXP gtracksummary_multitask(SEXP _expr, SEXP _intervals, SEXP _iterator_policy,
 		}
 	} catch (TGLException &e) {
 		rerror("%s", e.msg());
-	}
+    } catch (const bad_alloc &e) {
+        rerror("Out of memory");
+    }
 
 	rreturn(R_NilValue);
 }
@@ -253,15 +254,15 @@ SEXP gintervals_summary(SEXP _expr, SEXP _intervals, SEXP _iterator_policy, SEXP
 		GIntervalsFetcher1D *intervals1d = NULL;
 		GIntervalsFetcher2D *intervals2d = NULL;
 		iu.convert_rintervs(_intervals, &intervals1d, &intervals2d);
-		auto_ptr<GIntervalsFetcher1D> intervals1d_guard(intervals1d);
-		auto_ptr<GIntervalsFetcher2D> intervals2d_guard(intervals2d);
+		unique_ptr<GIntervalsFetcher1D> intervals1d_guard(intervals1d);
+		unique_ptr<GIntervalsFetcher2D> intervals2d_guard(intervals2d);
 		intervals1d->sort();
 		intervals2d->sort();
 		intervals2d->verify_no_overlaps(iu.get_chromkey());
 
 		scanner.begin(_expr, intervals1d, intervals2d, _iterator_policy, _band);
 
-		size_t num_intervals = scanner.get_iterator()->is_1d() ? intervals1d->size() : intervals2d->size();
+		uint64_t num_intervals = scanner.get_iterator()->is_1d() ? intervals1d->size() : intervals2d->size();
 		vector<IntervalSummary> summaries;
 
 		if (!num_intervals) 
@@ -284,14 +285,14 @@ SEXP gintervals_summary(SEXP _expr, SEXP _intervals, SEXP _iterator_policy, SEXP
 
 			if (scanner.get_iterator()->is_1d()) {
 				GIntervalsBigSet1D::begin_save(intervset_out.c_str(), iu, chromstats1d);
-				for (int chromid = 0; chromid < iu.get_chromkey().get_num_chroms(); ++chromid) {
+				for (int chromid = 0; (uint64_t)chromid < iu.get_chromkey().get_num_chroms(); ++chromid) {
 					if (intervals1d->size(chromid))
 						chroms1d.insert(chromid);
 				}
 			} else {
 				GIntervalsBigSet2D::begin_save(intervset_out.c_str(), iu, chromstats2d);
-				for (int chromid1 = 0; chromid1 < iu.get_chromkey().get_num_chroms(); ++chromid1) {
-					for (int chromid2 = 0; chromid2 < iu.get_chromkey().get_num_chroms(); ++chromid2) {
+				for (int chromid1 = 0; (uint64_t)chromid1 < iu.get_chromkey().get_num_chroms(); ++chromid1) {
+					for (int chromid2 = 0; (uint64_t)chromid2 < iu.get_chromkey().get_num_chroms(); ++chromid2) {
 						if (intervals2d->size(chromid1, chromid2))
 							chroms2d.insert(ChromPair(chromid1, chromid2));
 					}
@@ -302,7 +303,7 @@ SEXP gintervals_summary(SEXP _expr, SEXP _intervals, SEXP _iterator_policy, SEXP
 				float val = scanner.last_real(0);
 
 				if (cur_interval_idx != scanner.last_scope_idx()) {
-					size_t size;
+					uint64_t size;
 					char error_prefix[1000];
 
 					cur_interval_idx = scanner.last_scope_idx();
@@ -331,7 +332,7 @@ SEXP gintervals_summary(SEXP _expr, SEXP _intervals, SEXP _iterator_policy, SEXP
 				// interval has finished => calculate the percentile
 				if (scanner.get_iterator()->is_1d()) {
 					if (scanner.isend() || last_scope_interval1d.chromid != scanner.last_scope_interval1d().chromid) {
-						auto_ptr<GIntervalsFetcher1D> out_intervals(intervals1d->create_masked_copy(last_scope_interval1d.chromid));
+						unique_ptr<GIntervalsFetcher1D> out_intervals(intervals1d->create_masked_copy(last_scope_interval1d.chromid));
 
 						SEXP rintervals = build_rintervals_summary(out_intervals.get(), NULL, summaries, iu, false);
 						GIntervalsBigSet1D::save_chrom(intervset_out.c_str(), out_intervals.get(), rintervals, iu, chromstats1d);
@@ -340,7 +341,7 @@ SEXP gintervals_summary(SEXP _expr, SEXP _intervals, SEXP _iterator_policy, SEXP
 					}
 				} else {
 					if (scanner.isend() || !last_scope_interval2d.is_same_chrom(scanner.last_scope_interval2d())) {
-						auto_ptr<GIntervalsFetcher2D> out_intervals(
+						unique_ptr<GIntervalsFetcher2D> out_intervals(
 							intervals2d->create_masked_copy(last_scope_interval2d.chromid1(), last_scope_interval2d.chromid2()));
 
 						SEXP rintervals = build_rintervals_summary(NULL, out_intervals.get(), summaries, iu, false);
@@ -354,7 +355,7 @@ SEXP gintervals_summary(SEXP _expr, SEXP _intervals, SEXP _iterator_policy, SEXP
 			// finish saving (save skipped scope chromosomes + write meta)
 			if (scanner.get_iterator()->is_1d()) {
 				for (set<int>::const_iterator ichromid = chroms1d.begin(); ichromid != chroms1d.end(); ++ichromid) {
-					auto_ptr<GIntervalsFetcher1D> out_intervals(intervals1d->create_masked_copy(*ichromid));
+					unique_ptr<GIntervalsFetcher1D> out_intervals(intervals1d->create_masked_copy(*ichromid));
 
 					int size = intervals1d->size(*ichromid);
 					iu.verify_max_data_size(size, "Result", false);
@@ -369,7 +370,7 @@ SEXP gintervals_summary(SEXP _expr, SEXP _intervals, SEXP _iterator_policy, SEXP
 				GIntervalsBigSet1D::end_save(intervset_out.c_str(), zeroline, iu, chromstats1d);
 			} else {
 				for (set<ChromPair>::const_iterator ichrompair = chroms2d.begin(); ichrompair != chroms2d.end(); ++ichrompair)  {
-					auto_ptr<GIntervalsFetcher2D> out_intervals(intervals2d->create_masked_copy(ichrompair->chromid1, ichrompair->chromid2));
+					unique_ptr<GIntervalsFetcher2D> out_intervals(intervals2d->create_masked_copy(ichrompair->chromid1, ichrompair->chromid2));
 
 					int size = intervals2d->size(ichrompair->chromid1, ichrompair->chromid2);
 					iu.verify_max_data_size(size, "Result", false);
@@ -412,7 +413,9 @@ SEXP gintervals_summary(SEXP _expr, SEXP _intervals, SEXP _iterator_policy, SEXP
 		return answer;
 	} catch (TGLException &e) {
 		rerror("%s", e.msg());
-	}
+    } catch (const bad_alloc &e) {
+        rerror("Out of memory");
+    }
 	return R_NilValue;
 }
 
@@ -439,8 +442,8 @@ SEXP gbins_summary(SEXP _track_exprs, SEXP _breaks, SEXP _include_lowest, SEXP _
 		GIntervalsFetcher1D *intervals1d = NULL;
 		GIntervalsFetcher2D *intervals2d = NULL;
 		iu.convert_rintervs(_intervals, &intervals1d, &intervals2d);
-		auto_ptr<GIntervalsFetcher1D> intervals1d_guard(intervals1d);
-		auto_ptr<GIntervalsFetcher2D> intervals2d_guard(intervals2d);
+		unique_ptr<GIntervalsFetcher1D> intervals1d_guard(intervals1d);
+		unique_ptr<GIntervalsFetcher2D> intervals2d_guard(intervals2d);
 		intervals1d->sort();
 		intervals1d->unify_overlaps();
 		intervals2d->sort();
@@ -459,7 +462,7 @@ SEXP gbins_summary(SEXP _track_exprs, SEXP _breaks, SEXP _include_lowest, SEXP _
 
 		// pack the answer
 		SEXP answer, dim, dimnames;
-		rprotect(answer = allocVector(REALSXP, totalbins * NUM_COLS));
+		rprotect(answer = RSaneAllocVector(REALSXP, totalbins * NUM_COLS));
 
 		for (unsigned i = 0; i < totalbins; i++) {
 			REAL(answer)[totalbins * TOTAL_NAN_BINS + i] = summaries[i].num_bins - summaries[i].num_non_nan_bins;
@@ -471,14 +474,15 @@ SEXP gbins_summary(SEXP _track_exprs, SEXP _breaks, SEXP _include_lowest, SEXP _
 			REAL(answer)[totalbins * STDEV + i] = summaries[i].num_non_nan_bins > 1 ? summaries[i].get_stdev() : numeric_limits<double>::quiet_NaN();
 		}
 
-		rprotect(dim = allocVector(INTSXP, bins_manager.get_num_bin_finders() + 1));
-		rprotect(dimnames = allocVector(VECSXP, bins_manager.get_num_bin_finders() + 1));
+		rprotect(dim = RSaneAllocVector(INTSXP, bins_manager.get_num_bin_finders() + 1));
+		rprotect(dimnames = RSaneAllocVector(VECSXP, bins_manager.get_num_bin_finders() + 1));
 		bins_manager.set_dims(dim, dimnames);
 
 		SEXP dimname;
-		rprotect(dimname = allocVector(STRSXP, NUM_COLS));
+		rprotect(dimname = RSaneAllocVector(STRSXP, NUM_COLS));
 		for (unsigned i = 0; i < NUM_COLS; i++)
 			SET_STRING_ELT(dimname, i, mkChar(IntervalSummaryColNames[i]));
+
 		SET_VECTOR_ELT(dimnames, bins_manager.get_num_bin_finders(), dimname);
 		INTEGER(dim)[bins_manager.get_num_bin_finders()] = NUM_COLS;
 
@@ -487,7 +491,9 @@ SEXP gbins_summary(SEXP _track_exprs, SEXP _breaks, SEXP _include_lowest, SEXP _
 		return answer;
 	} catch (TGLException &e) {
 		rerror("%s", e.msg());
-	}
+    } catch (const bad_alloc &e) {
+        rerror("Out of memory");
+    }
 	return R_NilValue;
 }
 
