@@ -5,25 +5,24 @@
  *      Author: hoichman
  */
 
+#include <cstdint>
+#include <cmath>
 #include <fstream>
+#include <time.h>
 #include <unistd.h>
-#include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/time.h>
+#include <sys/types.h>
 
 #include "rdbinterval.h"
 #include "rdbutils.h"
 #include "TrackExpressionScanner.h"
 #include "GIntervalsBigSet1D.h"
 #include "GIntervalsBigSet2D.h"
-
-#ifdef __GXX_EXPERIMENTAL_CXX0X__
-#ifndef isnan
-#define isnan ::isnan
-#endif
-#endif
+#include "GTrackIntervalsFetcher1D.h"
+#include "GTrackIntervalsFetcher2D.h"
 
 using namespace std;
-using namespace __gnu_cxx;
 
 #include "strutil.h"
 
@@ -38,21 +37,14 @@ IntervUtils::IntervUtils(SEXP envir)
 	m_envir = envir;
 	m_num_planned_kids = 0;
 	m_multitasking = -1;
-	m_max_data_size = 0;
-	m_max_mem_usage = 0;
-	m_max_processes = 0;
-	m_max_processes2core = 0;
-	m_min_scope4process = 0;
-	m_quantile_edge_data_size = 0;
-	m_track_chunk_size = 0;
-	m_track_num_chunks = 0;
 	m_kid_intervals1d = NULL;
 	m_kid_intervals2d = NULL;
 
 	m_kids_intervals1d.clear();
 	m_kids_intervals2d.clear();
 
-	m_allgenome = findVar(install("ALLGENOME"), m_envir);
+	
+	m_allgenome = findVar(install("ALLGENOME"), findVar(install(".misha"), m_envir));
 
 	if (isNull(m_allgenome))
 		verror("ALLGENOME variable does not exist");
@@ -75,7 +67,7 @@ IntervUtils::IntervUtils(SEXP envir)
 		}
 	}
 
-	init_rnd_seed();
+    GenomeTrack::set_rnd_func(unif_rand);
 }
 
 IntervUtils::~IntervUtils()
@@ -86,22 +78,18 @@ IntervUtils::~IntervUtils()
 		delete *iinterv;
 }
 
-void IntervUtils::init_rnd_seed()
+bool IntervUtils::track_exists(const char *track_name)
 {
-	SEXP r_rnd_seed = GetOption(install("grnd.seed"), R_NilValue);
-	uint64_t rnd_seed;
+	SEXP all_track_names;
 
-	if (isReal(r_rnd_seed))
-		rnd_seed = (uint64_t)REAL(r_rnd_seed)[0];
-	else if (isInteger(r_rnd_seed))
-		rnd_seed = INTEGER(r_rnd_seed)[0];
-	else
-		rnd_seed = time(NULL);
-
-	if (!rnd_seed)
-		rnd_seed = time(NULL);
-
-	srand48(rnd_seed);
+	rprotect(all_track_names = findVar(install("GTRACKS"), findVar(install(".misha"), get_env())));
+	if (isString(all_track_names)) {
+		for (int i = 0; i < length(all_track_names); ++i) {
+			if (!strcmp(track_name, CHAR(STRING_ELT(all_track_names, i))))
+				return true;
+		}
+	}
+	return false;
 }
 
 void IntervUtils::get_all_genome_intervs(GIntervals &intervals) const
@@ -185,7 +173,7 @@ unsigned IntervUtils::get_rintervs_type_mask(SEXP rintervals, const char *error_
 				verror("%sNumber of rows in column %s differs than the number of rows in column %s", error_msg_prefix, GInterval::COL_NAMES[i - 1], GInterval::COL_NAMES[i]);
 		}
 
-		if (!isReal(starts) && !isInteger(starts) || !isReal(ends) && !isInteger(ends) || strands != R_NilValue && !isReal(strands) && !isInteger(strands))
+		if ((!isReal(starts) && !isInteger(starts)) || (!isReal(ends) && !isInteger(ends)) || (strands != R_NilValue && !isReal(strands) && !isInteger(strands)))
 			verror("%sInvalid format of intervals argument", error_msg_prefix);
 
 	} else if (type == INTERVS2D) {
@@ -199,7 +187,7 @@ unsigned IntervUtils::get_rintervs_type_mask(SEXP rintervals, const char *error_
 				verror("%sNumber of rows in column %s differs than the number of rows in column %s", error_msg_prefix, GInterval2D::COL_NAMES[i - 1], GInterval2D::COL_NAMES[i]);
 		}
 
-		if (!isReal(starts1) && !isInteger(starts1) || !isReal(ends1) && !isInteger(ends1) || !isReal(starts2) && !isInteger(starts2) || !isReal(ends2) && !isInteger(ends2))
+		if ((!isReal(starts1) && !isInteger(starts1)) || (!isReal(ends1) && !isInteger(ends1)) || (!isReal(starts2) && !isInteger(starts2)) || (!isReal(ends2) && !isInteger(ends2)))
 			verror("%sInvalid format of intervals argument", error_msg_prefix);
 	} else
 		verror("%sUnexpected intervals type", error_msg_prefix);
@@ -210,6 +198,7 @@ unsigned IntervUtils::get_rintervs_type_mask(SEXP rintervals, const char *error_
 unsigned IntervUtils::convert_rintervs(SEXP rintervals, GIntervalsFetcher1D **intervals1d, GIntervalsFetcher2D **intervals2d, bool null_if_interv_nonexist,
 									   const GenomeChromKey *chromkey, const char *error_msg_prefix, bool verify) const
 {
+	monitor_memusage();
 	if (intervals1d) 
 		*intervals1d = NULL;
 	if (intervals2d) 
@@ -219,9 +208,9 @@ unsigned IntervUtils::convert_rintervs(SEXP rintervals, GIntervalsFetcher1D **in
 			const char *full_interv_name = CHAR(STRING_ELT(rintervals, 0));
 
 			if (GIntervalsBigSet::isbig(full_interv_name, *this)) { // big intervals set?
-				SEXP meta = GIntervalsBigSet::get_meta(full_interv_name, *this);
+				SEXP meta = GIntervalsMeta::load_meta(interv2path(get_env(), full_interv_name).c_str());
 
-				if (GIntervalsBigSet1D::is1d(meta)) {
+				if (GIntervalsMeta1D::is1d(meta)) {
 					if (intervals1d) {
 						*intervals1d = new GIntervalsBigSet1D(full_interv_name, meta, *this);
 						if (intervals2d)
@@ -238,6 +227,48 @@ unsigned IntervUtils::convert_rintervs(SEXP rintervals, GIntervalsFetcher1D **in
 					}
 					verror("%sExpecting 1D intervals while receiving 2D intervals set %s", error_msg_prefix, full_interv_name);
 				}
+			} else if (GTrackIntervalsFetcher::isbig(full_interv_name, *this)) {
+				string trackpath = track2path(get_env(), full_interv_name);
+
+				if (access((trackpath + "/.meta").c_str(), R_OK) && errno == ENOENT)
+					GTrackIntervalsFetcher::create_track_meta(full_interv_name, *this);
+
+				SEXP meta = GIntervalsMeta::load_meta(trackpath.c_str());
+
+				if (GIntervalsMeta1D::is1d(meta)) {
+					if (intervals1d) {
+						GenomeTrack::Type track_type = GenomeTrack::get_type(trackpath.c_str(), get_chromkey(), true);
+
+						if (track_type == GenomeTrack::FIXED_BIN)
+							verror("%s is track of %s type which cannot be used as a replacement for intervals",
+								   full_interv_name, GenomeTrack::TYPE_NAMES[track_type]);
+						else if (track_type == GenomeTrack::SPARSE)
+							*intervals1d = new GTrackIntervalsFetcher1D<GenomeTrackSparse>(full_interv_name, meta, *this);
+						else if (track_type == GenomeTrack::ARRAYS)
+							*intervals1d = new GTrackIntervalsFetcher1D<GenomeTrackArrays>(full_interv_name, meta, *this);
+
+						if (intervals2d)
+							*intervals2d = new GIntervals2D();  // create an empty intervals set for convenience
+						return IntervUtils::INTERVS1D;
+					}
+					verror("%sExpecting 2D intervals while receiving 1D intervals set %s", error_msg_prefix, full_interv_name);
+				} else {
+					if (intervals2d) {
+						GenomeTrack::Type track_type = GenomeTrack::get_type(trackpath.c_str(), get_chromkey(), true);
+
+						if (track_type == GenomeTrack::RECTS)
+							*intervals2d = new GTrackIntervalsFetcher2D<GenomeTrackRectsRects>(full_interv_name, meta, *this);
+						else if (track_type == GenomeTrack::POINTS)
+							*intervals2d = new GTrackIntervalsFetcher2D<GenomeTrackRectsPoints>(full_interv_name, meta, *this);
+						else if (track_type == GenomeTrack::COMPUTED)
+							*intervals2d = new GTrackIntervalsFetcher2D<GenomeTrackComputed>(full_interv_name, meta, *this);
+
+						if (intervals1d)
+							*intervals1d = new GIntervals();  // create an empty intervals set for convenience
+						return IntervUtils::INTERVS2D;
+					}
+					verror("%sExpecting 1D intervals while receiving 2D intervals set %s", error_msg_prefix, full_interv_name);
+				}
 			}
 		}
 
@@ -266,6 +297,8 @@ unsigned IntervUtils::convert_rintervs(SEXP rintervals, GIntervalsFetcher1D **in
 SEXP IntervUtils::convert_rintervs(SEXP rintervals, GIntervals *intervals, GIntervals2D *intervals2d, bool null_if_interv_nonexist,
 								   const GenomeChromKey *chromkey, const char *error_msg_prefix, unsigned *pintervs_type_mask, bool verify) const
 {
+	monitor_memusage();
+
 	if (intervals)
 		intervals->clear();
 
@@ -283,7 +316,7 @@ SEXP IntervUtils::convert_rintervs(SEXP rintervals, GIntervals *intervals, GInte
 		SEXP gintervs;
 		bool interv_found = false;
 
-		rprotect(gintervs = findVar(install("GINTERVS"), m_envir));
+		rprotect(gintervs = findVar(install("GINTERVS"), findVar(install(".misha"),m_envir)));
 		if (isString(gintervs)) {
 			for (int iinterv = 0; iinterv < length(gintervs); ++iinterv) {
 				const char *interv = CHAR(STRING_ELT(gintervs, iinterv));
@@ -295,6 +328,9 @@ SEXP IntervUtils::convert_rintervs(SEXP rintervals, GIntervals *intervals, GInte
 		}
 
 		if (!interv_found) {
+			if (GTrackIntervalsFetcher::isbig(full_interv_name, *this))
+				verror("Tracks cannot be used as a substitute for intervals in this function");
+
 			if (null_if_interv_nonexist)
 				return R_NilValue;
 
@@ -306,7 +342,7 @@ SEXP IntervUtils::convert_rintervs(SEXP rintervals, GIntervals *intervals, GInte
 		if (!stat(path.c_str(), &stat_res) && S_ISDIR(stat_res.st_mode))
 			verror("%s is a big intervals set. Big intervals sets are not supported by the function.", full_interv_name);
 		rprotect(rintervals = RSaneUnserialize(path.c_str()));
-	}
+        }
 
 	if (TYPEOF(rintervals) == PROMSXP) {
 		if (PRENV(rintervals) == R_NilValue)
@@ -347,9 +383,9 @@ SEXP IntervUtils::convert_rintervs(SEXP rintervals, GIntervals *intervals, GInte
 		}
 
 		for (unsigned i = 0; i < num_intervals; i++) {
-			if (isFactor(chroms) && INTEGER(chroms)[i] < 0 ||
-				isReal(starts) && isnan(REAL(starts)[i]) || isReal(ends) && isnan(REAL(ends)[i]) ||
-				strands != R_NilValue && isReal(strands) && isnan(REAL(strands)[i]))
+			if ((isFactor(chroms) && INTEGER(chroms)[i] < 0) ||
+				(isReal(starts) && std::isnan(REAL(starts)[i])) || (isReal(ends) && std::isnan(REAL(ends)[i])) ||
+				(strands != R_NilValue && isReal(strands) && std::isnan(REAL(strands)[i])))
 				verror("%sInvalid format of interval at index %d", error_msg_prefix, i + 1);
 
 			const char *chrom = isString(chroms) ? CHAR(STRING_ELT(chroms, i)) : CHAR(STRING_ELT(chrom_levels, INTEGER(chroms)[i] - 1));
@@ -392,10 +428,10 @@ SEXP IntervUtils::convert_rintervs(SEXP rintervals, GIntervals *intervals, GInte
 		unsigned num_intervals = (unsigned)length(starts1);
 
 		for (unsigned i = 0; i < num_intervals; i++) {
-			if (isFactor(chroms1) && INTEGER(chroms1)[i] < 0 ||
-				isReal(starts1) && isnan(REAL(starts1)[i]) || isReal(ends1) && isnan(REAL(ends1)[i]) ||
-				isFactor(chroms2) && INTEGER(chroms2)[i] < 0 ||
-				isReal(starts2) && isnan(REAL(starts2)[i]) || isReal(ends2) && isnan(REAL(ends2)[i]))
+			if ((isFactor(chroms1) && INTEGER(chroms1)[i] < 0) ||
+				(isReal(starts1) && std::isnan(REAL(starts1)[i])) || (isReal(ends1) && std::isnan(REAL(ends1)[i])) ||
+				(isFactor(chroms2) && INTEGER(chroms2)[i] < 0) ||
+				(isReal(starts2) && std::isnan(REAL(starts2)[i])) || (isReal(ends2) && std::isnan(REAL(ends2)[i])))
 				verror("%sInvalid format of interval at index %d", error_msg_prefix, i + 1);
 
 			const char *chrom1 = isString(chroms1) ? CHAR(STRING_ELT(chroms1, i)) : CHAR(STRING_ELT(chrom_levels1, INTEGER(chroms1)[i] - 1));
@@ -420,6 +456,8 @@ SEXP IntervUtils::convert_rintervs(SEXP rintervals, GIntervals *intervals, GInte
 
 SEXP IntervUtils::convert_intervs(GIntervalsFetcher1D *intervals, unsigned num_cols, bool null_if_empty, bool use_original_index) const
 {
+	monitor_memusage();
+
 	if (null_if_empty && !intervals->size())
 		return R_NilValue;
 
@@ -430,22 +468,17 @@ SEXP IntervUtils::convert_intervs(GIntervalsFetcher1D *intervals, unsigned num_c
 	SEXP row_names;
 	SEXP col_names;
 
-	rprotect(answer = allocVector(VECSXP, num_cols));
-
-	SET_VECTOR_ELT(answer, GInterval::CHROM, (chroms_idx = allocVector(INTSXP, intervals->size())));
-	SET_VECTOR_ELT(answer, GInterval::START, (starts = allocVector(REALSXP, intervals->size())));
-	SET_VECTOR_ELT(answer, GInterval::END, (ends = allocVector(REALSXP, intervals->size())));
-
-	setAttrib(chroms_idx, R_LevelsSymbol, (chroms = allocVector(STRSXP, num_chroms)));
-	setAttrib(chroms_idx, R_ClassSymbol, mkString("factor"));
-
-	setAttrib(answer, R_NamesSymbol, (col_names = allocVector(STRSXP, num_cols)));
-	setAttrib(answer, R_ClassSymbol, mkString("data.frame"));
-	setAttrib(answer, R_RowNamesSymbol, (row_names = allocVector(INTSXP, intervals->size())));
+	rprotect(answer = RSaneAllocVector(VECSXP, num_cols));
+    rprotect(chroms_idx = RSaneAllocVector(INTSXP, intervals->size()));
+    rprotect(starts = RSaneAllocVector(REALSXP, intervals->size()));
+    rprotect(ends = RSaneAllocVector(REALSXP, intervals->size()));
+    rprotect(chroms = RSaneAllocVector(STRSXP, num_chroms));
+    rprotect(col_names = RSaneAllocVector(STRSXP, num_cols));
+    rprotect(row_names = RSaneAllocVector(INTSXP, intervals->size()));
 
 	for (intervals->begin_iter(); !intervals->isend(); intervals->next()) {
 		const GInterval &interval = intervals->cur_interval();
-		size_t index = use_original_index ? get_orig_interv_idx(interval) : intervals->iter_index();
+		uint64_t index = use_original_index ? get_orig_interv_idx(interval) : intervals->iter_index();
 
 		INTEGER(chroms_idx)[index] = interval.chromid + 1;
 		REAL(starts)[index] = interval.start;
@@ -459,11 +492,24 @@ SEXP IntervUtils::convert_intervs(GIntervalsFetcher1D *intervals, unsigned num_c
 	for (int i = 0; i < GInterval::NUM_COLS; i++)
 		SET_STRING_ELT(col_names, i, mkChar(GInterval::COL_NAMES[i]));
 
+    setAttrib(chroms_idx, R_LevelsSymbol, chroms);
+    setAttrib(chroms_idx, R_ClassSymbol, mkString("factor"));
+
+    SET_VECTOR_ELT(answer, GInterval::CHROM, chroms_idx);
+    SET_VECTOR_ELT(answer, GInterval::START, starts);
+    SET_VECTOR_ELT(answer, GInterval::END, ends);
+
+    setAttrib(answer, R_NamesSymbol, col_names);
+    setAttrib(answer, R_ClassSymbol, mkString("data.frame"));
+    setAttrib(answer, R_RowNamesSymbol, row_names);
+
 	return answer;
 }
 
 SEXP IntervUtils::convert_intervs(GIntervalsFetcher2D *intervals, unsigned num_cols, bool null_if_empty, bool use_original_index) const
 {
+	monitor_memusage();
+
 	if (null_if_empty && !intervals->size())
 		return R_NilValue;
 
@@ -474,46 +520,58 @@ SEXP IntervUtils::convert_intervs(GIntervalsFetcher2D *intervals, unsigned num_c
 	SEXP row_names;
 	SEXP col_names;
 
-	rprotect(answer = allocVector(VECSXP, num_cols));
-
-	SET_VECTOR_ELT(answer, GInterval2D::CHROM1, (chroms_idx1 = allocVector(INTSXP, intervals->size())));
-	SET_VECTOR_ELT(answer, GInterval2D::START1, (starts1 = allocVector(REALSXP, intervals->size())));
-	SET_VECTOR_ELT(answer, GInterval2D::END1, (ends1 = allocVector(REALSXP, intervals->size())));
-	SET_VECTOR_ELT(answer, GInterval2D::CHROM2, (chroms_idx2 = allocVector(INTSXP, intervals->size())));
-	SET_VECTOR_ELT(answer, GInterval2D::START2, (starts2 = allocVector(REALSXP, intervals->size())));
-	SET_VECTOR_ELT(answer, GInterval2D::END2, (ends2 = allocVector(REALSXP, intervals->size())));
-
-	setAttrib(chroms_idx1, R_LevelsSymbol, (chroms1 = allocVector(STRSXP, num_chroms)));
-	setAttrib(chroms_idx1, R_ClassSymbol, mkString("factor"));
-	setAttrib(chroms_idx2, R_LevelsSymbol, (chroms2 = allocVector(STRSXP, num_chroms)));
-	setAttrib(chroms_idx2, R_ClassSymbol, mkString("factor"));
-
-	setAttrib(answer, R_NamesSymbol, (col_names = allocVector(STRSXP, num_cols)));
-	setAttrib(answer, R_ClassSymbol, mkString("data.frame"));
-	setAttrib(answer, R_RowNamesSymbol, (row_names = allocVector(INTSXP, intervals->size())));
+	rprotect(answer = RSaneAllocVector(VECSXP, num_cols));
+    rprotect(chroms1 = RSaneAllocVector(STRSXP, num_chroms));
+    rprotect(starts1 = RSaneAllocVector(REALSXP, intervals->size()));
+    rprotect(ends1 = RSaneAllocVector(REALSXP, intervals->size()));
+    rprotect(chroms_idx1 = RSaneAllocVector(INTSXP, intervals->size()));
+    rprotect(chroms_idx2 = RSaneAllocVector(INTSXP, intervals->size()));
+    rprotect(starts2 = RSaneAllocVector(REALSXP, intervals->size()));
+    rprotect(ends2 = RSaneAllocVector(REALSXP, intervals->size()));
+    rprotect(chroms1 = RSaneAllocVector(STRSXP, num_chroms));
+    rprotect(chroms2 = RSaneAllocVector(STRSXP, num_chroms));
+    rprotect(col_names = RSaneAllocVector(STRSXP, num_cols));
+    rprotect(row_names = RSaneAllocVector(INTSXP, intervals->size()));
 
 	for (intervals->begin_iter(); !intervals->isend(); intervals->next()) {
 		const GInterval2D &interval = intervals->cur_interval();
-		size_t index = use_original_index ? get_orig_interv_idx(interval) : intervals->iter_index();
+		uint64_t index = use_original_index ? get_orig_interv_idx(interval) : intervals->iter_index();
 
 		INTEGER(chroms_idx1)[index] = interval.chromid1() + 1;
 		REAL(starts1)[index] = interval.start1();
-		REAL(ends1)[index] = interval.end1();
-		INTEGER(chroms_idx2)[index] = interval.chromid2() + 1;
-		REAL(starts2)[index] = interval.start2();
-		REAL(ends2)[index] = interval.end2();
+        REAL(ends1)[index] = interval.end1();
+        INTEGER(chroms_idx2)[index] = interval.chromid2() + 1;
+        REAL(starts2)[index] = interval.start2();
+        REAL(ends2)[index] = interval.end2();
 		INTEGER(row_names)[index] = index + 1;
 	}
 
-	for (unsigned id = 0; id < (unsigned)num_chroms; ++id) {
-		SET_STRING_ELT(chroms1, id, mkChar(m_chrom_key.id2chrom(id).c_str()));
-		SET_STRING_ELT(chroms2, id, mkChar(m_chrom_key.id2chrom(id).c_str()));
-	}
+    for (unsigned id = 0; id < (unsigned)num_chroms; ++id) {
+        SET_STRING_ELT(chroms1, id, mkChar(m_chrom_key.id2chrom(id).c_str()));
+        SET_STRING_ELT(chroms2, id, mkChar(m_chrom_key.id2chrom(id).c_str()));
+    }
 
-	for (int i = 0; i < GInterval2D::NUM_COLS; i++)
-		SET_STRING_ELT(col_names, i, mkChar(GInterval2D::COL_NAMES[i]));
+    for (int i = 0; i < GInterval2D::NUM_COLS; i++)
+        SET_STRING_ELT(col_names, i, mkChar(GInterval2D::COL_NAMES[i]));
 
-	return answer;
+
+    setAttrib(chroms_idx1, R_LevelsSymbol, chroms1);
+    setAttrib(chroms_idx1, R_ClassSymbol, mkString("factor"));
+    setAttrib(chroms_idx2, R_LevelsSymbol, chroms2);
+    setAttrib(chroms_idx2, R_ClassSymbol, mkString("factor"));
+
+    SET_VECTOR_ELT(answer, GInterval2D::CHROM1, chroms_idx1);
+    SET_VECTOR_ELT(answer, GInterval2D::START1, starts1);
+    SET_VECTOR_ELT(answer, GInterval2D::END1, ends1);
+    SET_VECTOR_ELT(answer, GInterval2D::CHROM2, chroms_idx2);
+    SET_VECTOR_ELT(answer, GInterval2D::START2, starts2);
+    SET_VECTOR_ELT(answer, GInterval2D::END2, ends2);
+
+    setAttrib(answer, R_NamesSymbol, col_names);
+    setAttrib(answer, R_ClassSymbol, mkString("data.frame"));
+    setAttrib(answer, R_RowNamesSymbol, row_names);
+
+    return answer;
 }
 
 void IntervUtils::convert_rchain_intervs(SEXP rchain, ChainIntervals &chain_intervs, vector<string> &src_id2chrom)
@@ -548,14 +606,14 @@ void IntervUtils::convert_rchain_intervs(SEXP rchain, ChainIntervals &chain_inte
 	if (!isReal(src_starts) && !isInteger(src_starts))
 		verror("Invalid format of intervals argument");
 
-	hash_map<string, int> src_chrom2id;
+	unordered_map<string, int> src_chrom2id;
 
 	for (unsigned i = 0; i < intervs.size(); i++) {
-		if (isFactor(src_chroms) && INTEGER(src_chroms)[i] < 0 || isReal(src_starts) && isnan(REAL(src_starts)[i]))
+		if ((isFactor(src_chroms) && INTEGER(src_chroms)[i] < 0) || (isReal(src_starts) && std::isnan(REAL(src_starts)[i])))
 			verror("Invalid format of interval at index %d", i + 1);
 
 		const char *src_chrom = isString(src_chroms) ? CHAR(STRING_ELT(src_chroms, i)) : CHAR(STRING_ELT(src_chrom_levels, INTEGER(src_chroms)[i] - 1));
-		hash_map<string, int>::const_iterator isrc_chrom2id = src_chrom2id.find(src_chrom);
+		unordered_map<string, int>::const_iterator isrc_chrom2id = src_chrom2id.find(src_chrom);
 		int src_chromid;
 
 		if (isrc_chrom2id == src_chrom2id.end()) {
@@ -586,11 +644,9 @@ SEXP IntervUtils::convert_chain_intervs(const ChainIntervals &chain_intervs, vec
 	SEXP col_names = getAttrib(answer, R_NamesSymbol);
 	unsigned num_src_chroms = src_id2chrom.size();
 
-	SET_VECTOR_ELT(answer, ChainInterval::CHROM_SRC, (src_chroms_idx = allocVector(INTSXP, chain_intervs.size())));
-	SET_VECTOR_ELT(answer, ChainInterval::START_SRC, (src_starts = allocVector(REALSXP, chain_intervs.size())));
-
-	setAttrib(src_chroms_idx, R_LevelsSymbol, (src_chroms = allocVector(STRSXP, num_src_chroms)));
-	setAttrib(src_chroms_idx, R_ClassSymbol, mkString("factor"));
+    rprotect(src_chroms_idx = RSaneAllocVector(INTSXP, chain_intervs.size()));
+    rprotect(src_starts = RSaneAllocVector(REALSXP, chain_intervs.size()));
+    rprotect(src_chroms = RSaneAllocVector(STRSXP, num_src_chroms));
 
 	for (ChainIntervals::const_iterator iinterval = chain_intervs.begin(); iinterval != chain_intervs.end(); ++iinterval) {
 		INTEGER(src_chroms_idx)[iinterval - chain_intervs.begin()] = iinterval->chromid_src + 1;
@@ -603,6 +659,12 @@ SEXP IntervUtils::convert_chain_intervs(const ChainIntervals &chain_intervs, vec
 	for (int i = 0; i < ChainInterval::NUM_COLS; i++)
 		SET_STRING_ELT(col_names, i, mkChar(ChainInterval::COL_NAMES[i]));
 
+    setAttrib(src_chroms_idx, R_LevelsSymbol, src_chroms);
+    setAttrib(src_chroms_idx, R_ClassSymbol, mkString("factor"));
+
+    SET_VECTOR_ELT(answer, ChainInterval::CHROM_SRC, src_chroms_idx);
+    SET_VECTOR_ELT(answer, ChainInterval::START_SRC, src_starts);
+
 	return answer;
 }
 
@@ -611,7 +673,7 @@ DiagonalBand IntervUtils::convert_band(SEXP rband)
 	if (isNull(rband))
 		return DiagonalBand();
 
-	if (!isReal(rband) && !isInteger(rband) || length(rband) != 2)
+	if ((!isReal(rband) && !isInteger(rband)) || length(rband) != 2)
 		verror("Invalid format of band argument");
 
 	int d1 = isReal(rband) ? (int)(REAL(rband)[0] > 0 ? REAL(rband)[0] + 0.5 : REAL(rband)[0] - 0.5) : INTEGER(rband)[0];
@@ -625,20 +687,22 @@ DiagonalBand IntervUtils::convert_band(SEXP rband)
 
 SEXP IntervUtils::create_data_frame(int numrows, int numcols, SEXP attrs_src)
 {
-	SEXP answer;
-	SEXP row_names;
-	rprotect(answer = allocVector(VECSXP, numcols));
+	SEXP answer, row_names, col_names;
 
-	if (attrs_src != R_NilValue) 
-		copyMostAttrib(attrs_src, answer);
-
-	setAttrib(answer, R_NamesSymbol, allocVector(STRSXP, numcols));
-	setAttrib(answer, R_ClassSymbol, mkString("data.frame"));
-	setAttrib(answer, R_RowNamesSymbol, (row_names = allocVector(INTSXP, numrows)));
+	rprotect(answer = RSaneAllocVector(VECSXP, numcols));
+    rprotect(col_names = RSaneAllocVector(STRSXP, numcols));
+    rprotect(row_names = RSaneAllocVector(INTSXP, numrows));
 
 	for (int i = 0; i < numrows; ++i)
 		INTEGER(row_names)[i] = i + 1;
 
+    if (attrs_src != R_NilValue) 
+        copyMostAttrib(attrs_src, answer);
+
+    setAttrib(answer, R_NamesSymbol, col_names);
+    setAttrib(answer, R_ClassSymbol, mkString("data.frame"));
+    setAttrib(answer, R_RowNamesSymbol, row_names);
+
 	return answer;
 }
 
@@ -646,8 +710,11 @@ void IntervUtils::define_data_frame_cols(SEXP src, vector<SEXP> &src_cols, SEXP
 {
 	SEXP src_class = getAttrib(src, R_ClassSymbol);
 
-	if (isNull(src_class) || !isString(src_class) || length(src_class) != 1 || strcmp(CHAR(STRING_ELT(src_class, 0)), "data.frame"))
-		verror("Copied object is not a data frame");
+	if (isNull(src_class) || !isString(src_class) ||
+		(!(length(src_class) == 1 && !strcmp(CHAR(STRING_ELT(src_class, 0)), "data.frame")) &&
+		!(length(src_class) == 3 && !strcmp(CHAR(STRING_ELT(src_class, 0)), "tbl_df")  &&
+          !strcmp(CHAR(STRING_ELT(src_class, 1)), "tbl") && !strcmp(CHAR(STRING_ELT(src_class, 2)), "data.frame"))))
+		verror("Copied object is not a data frame or tibble");
 
 	if (length(tgt) < length(src) + tgt_col_offset)
 		verror("Attempt to copy data frame columns beyond the valid size");
@@ -660,27 +727,31 @@ void IntervUtils::define_data_frame_cols(SEXP src, vector<SEXP> &src_cols, SEXP
 		verror("Invalid source data frame for a copy");
 
 	src_cols.resize(length(src));
-	if (tgt_cols.size() < length(tgt) + tgt_col_offset) 
+	if (tgt_cols.size() < (uint64_t)(length(tgt) + tgt_col_offset)){ 
 		tgt_cols.resize(length(tgt) + tgt_col_offset);
+	}
 
 	for (int col = 0; col < length(src); ++col) {
 		SEXP src_col = VECTOR_ELT(src, col);
 		SEXP tgt_col;
 
+        rprotect(tgt_col = RSaneAllocVector(TYPEOF(src_col), numrows));
+
 		if (!isInteger(src_col) && !isReal(src_col) && !isLogical(src_col) && !isString(src_col) && !isFactor(src_col))
 			verror("Unsupported type found in a data frame: %s", type2char(TYPEOF(src_col)));
 
-		SET_VECTOR_ELT(tgt, col + tgt_col_offset, (tgt_col = allocVector(TYPEOF(src_col), numrows)));
 		copyMostAttrib(src_col, tgt_col);
 		SET_STRING_ELT(tgt_colnames, col + tgt_col_offset, STRING_ELT(src_colnames, col));
 		src_cols[col] = src_col;
 		tgt_cols[col + tgt_col_offset] = tgt_col;
-	}
+
+        SET_VECTOR_ELT(tgt, col + tgt_col_offset, tgt_col);
+    }
 }
 
 void IntervUtils::copy_data_frame_row(const vector<SEXP> &src_cols, int src_row, const vector<SEXP> &tgt_cols, int tgt_row, int tgt_col_offset)
 {
-	for (size_t col = 0; col < src_cols.size(); ++col) {
+	for (uint64_t col = 0; col < src_cols.size(); ++col) {
 		SEXP src_col = src_cols[col];
 		SEXP tgt_col = tgt_cols[col + tgt_col_offset];
 
@@ -697,7 +768,7 @@ void IntervUtils::copy_data_frame_row(const vector<SEXP> &src_cols, int src_row,
 
 void IntervUtils::copy_data_frame_rows(const vector<SEXP> &src_cols, int src_row, int num_rows, const vector<SEXP> &tgt_cols, int tgt_row, int tgt_col_offset)
 {
-	for (int col = 0; col < src_cols.size(); ++col) {
+	for (uint64_t col = 0; col < src_cols.size(); ++col) {
 		SEXP src_col = src_cols[col];
 		SEXP tgt_col = tgt_cols[col + tgt_col_offset];
 
@@ -740,7 +811,7 @@ void IntervUtils::set_data_frame_val_nan(const vector<SEXP> &tgt_cols, int tgt_r
 void IntervUtils::restrict_bins(int64_t maxbins, GIntervals &intervals, unsigned binsize) const
 {
 	for (GIntervals::const_iterator iinterval = intervals.begin(); iinterval != intervals.end(); ++iinterval) {
-		int64_t bins = max(0L, (int64_t)ceil(iinterval->end / binsize) - (int64_t)(iinterval->start / binsize));
+		int64_t bins = max((int64_t)0, (int64_t)ceil(iinterval->end / binsize) - (int64_t)(iinterval->start / binsize));
 
 		if (bins > maxbins)
 			verror("The interval %s [%ld, %ld) covers too wide range of samples that might cause memory allocation failure.\n"
@@ -748,7 +819,7 @@ void IntervUtils::restrict_bins(int64_t maxbins, GIntervals &intervals, unsigned
 	}
 }
 
-bool IntervUtils::get_multitasking()
+bool IntervUtils::get_multitasking() const
 {
 	if (m_multitasking < 0) {
 		SEXP r_multitasking = GetOption(install("gmultitasking"), R_NilValue);
@@ -761,7 +832,7 @@ bool IntervUtils::get_multitasking()
 	return (bool)m_multitasking;
 }
 
-uint64_t IntervUtils::get_max_processes()
+uint64_t IntervUtils::get_max_processes() const
 {
 	if (!m_max_processes) {
 		SEXP r_max_processes = GetOption(install("gmax.processes"), R_NilValue);
@@ -778,7 +849,7 @@ uint64_t IntervUtils::get_max_processes()
 	return m_max_processes;
 }
 
-uint64_t IntervUtils::get_max_processes2core()
+uint64_t IntervUtils::get_max_processes2core() const
 {
 	if (!m_max_processes2core) {
 		SEXP r_max_processes2core = GetOption(install("gmax.processes2core"), R_NilValue);
@@ -795,7 +866,7 @@ uint64_t IntervUtils::get_max_processes2core()
 	return m_max_processes2core;
 }
 
-uint64_t IntervUtils::get_min_scope4process()
+uint64_t IntervUtils::get_min_scope4process() const
 {
 	if (!m_min_scope4process) {
 		SEXP r_min_scope4process = GetOption(install("gmin.scope4process"), R_NilValue);
@@ -810,7 +881,7 @@ uint64_t IntervUtils::get_min_scope4process()
 	return m_min_scope4process;
 }
 
-uint64_t IntervUtils::get_max_data_size()
+uint64_t IntervUtils::get_max_data_size() const
 {
 	if (!m_max_data_size) {
 		SEXP r_max_data_size = GetOption(install("gmax.data.size"), R_NilValue);
@@ -825,7 +896,7 @@ uint64_t IntervUtils::get_max_data_size()
 	return m_max_data_size;
 }
 
-uint64_t IntervUtils::get_max_mem_usage()
+uint64_t IntervUtils::get_max_mem_usage() const
 {
 	if (!m_max_mem_usage) {
 		SEXP r_max_mem_usage = GetOption(install("gmax.mem.usage"), R_NilValue);
@@ -840,7 +911,7 @@ uint64_t IntervUtils::get_max_mem_usage()
 	return m_max_mem_usage;
 }
 
-uint64_t IntervUtils::get_big_intervals_size()
+uint64_t IntervUtils::get_big_intervals_size() const
 {
 	if (!m_big_intervals_size) {
 		SEXP r_big_intervals_size = GetOption(install("gbig.intervals.size"), R_NilValue);
@@ -856,7 +927,7 @@ uint64_t IntervUtils::get_big_intervals_size()
 	return m_big_intervals_size;
 }
 
-uint64_t IntervUtils::get_quantile_edge_data_size()
+uint64_t IntervUtils::get_quantile_edge_data_size() const
 {
 	if (!m_quantile_edge_data_size) {
 		SEXP r_quantile_edge_data_size = GetOption(install("gquantile.edge.data.size"), R_NilValue);
@@ -871,7 +942,7 @@ uint64_t IntervUtils::get_quantile_edge_data_size()
 	return m_quantile_edge_data_size;
 }
 
-uint64_t IntervUtils::get_track_chunk_size()
+uint64_t IntervUtils::get_track_chunk_size() const
 {
 	if (!m_track_chunk_size) {
 		SEXP r_track_chunk_size = GetOption(install("gtrack.chunk.size"), R_NilValue);
@@ -886,7 +957,7 @@ uint64_t IntervUtils::get_track_chunk_size()
 	return m_track_chunk_size;
 }
 
-uint64_t IntervUtils::get_track_num_chunks()
+uint64_t IntervUtils::get_track_num_chunks() const
 {
 	if (!m_track_num_chunks) {
 		SEXP r_track_num_chunks = GetOption(install("gtrack.num.chunks"), R_NilValue);
@@ -941,7 +1012,7 @@ int IntervUtils::prepare4multitasking(GIntervalsFetcher1D *scope1d, GIntervalsFe
 	if (scope1d && !scope1d->size() && scope2d && !scope2d->size()) 
 		return 0;
 
-	if (scope1d && scope1d->size() && scope2d && scope2d->size() || !scope1d && !scope2d) 
+	if ((scope1d && scope1d->size() && scope2d && scope2d->size()) || (!scope1d && !scope2d)) 
 		verror("Cannot determine iterator policy");
 
 	int num_cores = max(1, (int)sysconf(_SC_NPROCESSORS_ONLN));
@@ -975,7 +1046,7 @@ int IntervUtils::prepare4multitasking(GIntervalsFetcher1D *scope1d, GIntervalsFe
 					}
 
 					// should we allocate a new process?
-					if (kid_range > get_min_scope4process() && range > get_min_scope4process() && kid_range + chrom_range >= range / num_avail_kids) {
+					if ((uint64_t)kid_range > get_min_scope4process() && (uint64_t)range > get_min_scope4process() && (uint64_t)(kid_range + chrom_range) >= (uint64_t)(range / num_avail_kids)) {
 						--num_avail_kids;
 						kid_range = 0;
 						m_kids_intervals1d.push_back(new GIntervals());
@@ -1003,7 +1074,7 @@ int IntervUtils::prepare4multitasking(GIntervalsFetcher1D *scope1d, GIntervalsFe
 
 				if (kid_range && num_avail_kids) {
 					// should we allocate a new process?
-					if (kid_range > get_min_scope4process() && range > get_min_scope4process() && kid_range + chrom_range >= range / num_avail_kids) {
+					if ((uint64_t)kid_range > get_min_scope4process() && (uint64_t)range > get_min_scope4process() && kid_range + chrom_range >= range / num_avail_kids) {
 						--num_avail_kids;
 						kid_range = 0;
 						m_kids_intervals1d.push_back(scope1d->create_masked_copy(chromids_mask));
@@ -1026,7 +1097,7 @@ int IntervUtils::prepare4multitasking(GIntervalsFetcher1D *scope1d, GIntervalsFe
 //int idx = 0;
 //for (vector<GIntervalsFetcher1D *>::iterator i = m_kids_intervals1d.begin(); i != m_kids_intervals1d.end(); ++i){
 //GIntervalsFetcher1D *fetcher = *i;
-//printf("Kid %d:\n", ++idx);
+//REprintf("Kid %d:\n", ++idx);
 //GIntervals all_genome;
 //get_all_genome_intervs(all_genome);
 //
@@ -1034,11 +1105,11 @@ int IntervUtils::prepare4multitasking(GIntervalsFetcher1D *scope1d, GIntervalsFe
 //for (GIntervals::const_iterator iinterv = all_genome.begin(); iinterv != all_genome.end(); ++iinterv) {
 //int64_t chrom_range = fetcher->range(iinterv->chromid);
 //if (chrom_range) {
-//printf("\t%s\trange %ld\n", id2chrom(iinterv->chromid).c_str(), chrom_range);
+//REprintf("\t%s\trange %ld\n", id2chrom(iinterv->chromid).c_str(), chrom_range);
 //total_range += chrom_range;
 //}
 //}
-//printf("total range: %ld\n\n", total_range);
+//REprintf("total range: %ld\n\n", total_range);
 //}
 
 	} else if (scope2d && scope2d->size()) {
@@ -1116,7 +1187,7 @@ int IntervUtils::prepare4multitasking(GIntervalsFetcher1D *scope1d, GIntervalsFe
 //int idx = 0;
 //for (vector<GIntervalsFetcher2D *>::iterator i = m_kids_intervals2d.begin(); i != m_kids_intervals2d.end(); ++i){
 //GIntervalsFetcher2D *fetcher = *i;
-//printf("Kid %d:\n", ++idx);
+//REprintf("Kid %d:\n", ++idx);
 //GIntervals2D all_genome;
 //get_all_genome_intervs(all_genome);
 //
@@ -1125,10 +1196,10 @@ int IntervUtils::prepare4multitasking(GIntervalsFetcher1D *scope1d, GIntervalsFe
 //double chrom_surface = fetcher->surface(iinterv->chromid1(), iinterv->chromid2());
 //if (chrom_surface) {
 //total_surface += chrom_surface;
-//printf("\t%s\t%s\tsurface %g\n", id2chrom(iinterv->chromid1()).c_str(), id2chrom(iinterv->chromid2()).c_str(), chrom_surface);
+//REprintf("\t%s\t%s\tsurface %g\n", id2chrom(iinterv->chromid1()).c_str(), id2chrom(iinterv->chromid2()).c_str(), chrom_surface);
 //}
 //}
-//printf("total_surface: %g\n\n", total_surface);
+//REprintf("total_surface: %g\n\n", total_surface);
 //}
 	}
 
@@ -1136,10 +1207,10 @@ int IntervUtils::prepare4multitasking(GIntervalsFetcher1D *scope1d, GIntervalsFe
 	return m_num_planned_kids;
 }
 
-bool IntervUtils::distribute_task(size_t res_const_size,    // data size in bytes for all the result
-								  size_t res_record_size)   // size in bytes per datum in the result
+bool IntervUtils::distribute_task(uint64_t res_const_size,    // data size in bytes for all the result
+								  uint64_t res_record_size)   // size in bytes per datum in the result
 {
-	size_t max_res_size = get_max_data_size() * res_record_size + m_num_planned_kids * res_const_size;
+	uint64_t max_res_size = get_max_data_size() * res_record_size + m_num_planned_kids * res_const_size;
 
 	rdb::prepare4multitasking(res_const_size, res_record_size, max_res_size, get_max_mem_usage(), m_num_planned_kids);
 
@@ -1185,13 +1256,16 @@ ChainIntervals::const_iterator ChainIntervals::map_interval(const GInterval &src
 {
 	tgt_intervs.clear();
 
+//for (auto i = begin(); i < end(); ++i)
+//REprintf("chrom %d, coord %ld\n", i->chromid_src, i->start_src);
+
 	if (empty())
 		return end();
 
-	if (front().chromid_src > src_interval.chromid || front().chromid_src == src_interval.chromid && front().start_src >= src_interval.end)
+	if (front().chromid_src > src_interval.chromid || (front().chromid_src == src_interval.chromid && front().start_src >= src_interval.end))
 		return begin();
 
-	if (back().chromid_src < src_interval.chromid || front().chromid_src == src_interval.chromid && back().start_src + back().end - back().start <= src_interval.start)
+	if (back().chromid_src < src_interval.chromid || (back().chromid_src == src_interval.chromid && back().start_src + back().end - back().start <= src_interval.start))
 		return end() - 1;
 
 	if (check_first_overlap_src(hint, src_interval))
@@ -1211,8 +1285,7 @@ ChainIntervals::const_iterator ChainIntervals::map_interval(const GInterval &src
 			return add2tgt(imid_interval, src_interval, tgt_intervs);
 
 		// is mid_interval < interval?
-		if (imid_interval->chromid_src < src_interval.chromid ||
-				imid_interval->chromid_src == src_interval.chromid && imid_interval->start_src < src_interval.start)
+		if (imid_interval->chromid_src < src_interval.chromid || (imid_interval->chromid_src == src_interval.chromid && imid_interval->start_src < src_interval.start))
 			istart_interval = imid_interval;
 		else
 			iend_interval = imid_interval;
